Class: String
- Inherits:
-
Object
- Object
- String
- Defined in:
- lib/bcl/core_ext.rb
Overview
******************************************************************************* OpenStudio®, Copyright © 2008-2021, Alliance for Sustainable Energy, LLC. All rights reserved. Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
(1) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
(2) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
(3) Neither the name of the copyright holder nor the names of any contributors may be used to endorse or promote products derived from this software without specific prior written permission from the respective party.
(4) Other than as required in clauses (1) and (2), distributions in any form of modifications or other derivative works may not use the “OpenStudio” trademark, “OS”, “os”, or any other confusingly similar designation without specific prior written permission from Alliance for Sustainable Energy, LLC.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) AND ANY CONTRIBUTORS “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER(S), ANY CONTRIBUTORS, THE UNITED STATES GOVERNMENT, OR THE UNITED STATES DEPARTMENT OF ENERGY, NOR ANY OF THEIR EMPLOYEES, BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. *******************************************************************************
Instance Method Summary collapse
-
#titleize ⇒ Object
simple method to create titles – very custom to catch known inflections.
- #to_bool ⇒ Object
- #to_underscore ⇒ Object
Instance Method Details
#titleize ⇒ Object
simple method to create titles – very custom to catch known inflections
57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 |
# File 'lib/bcl/core_ext.rb', line 57 def titleize arr = ['a', 'an', 'the', 'by', 'to'] upcase_arr = ['DX', 'EDA', 'AEDG', 'LPD', 'COP'] r = tr('_', ' ').gsub(/\w+/) do |match| match_result = match if upcase_arr.include?(match.upcase) match_result = upcase_arr[upcase_arr.index(match.upcase)] elsif arr.include?(match) match_result = match else match_result = match.capitalize end match_result end r end |
#to_bool ⇒ Object
49 50 51 52 53 54 |
# File 'lib/bcl/core_ext.rb', line 49 def to_bool return true if self == true || self =~ /(true|t|yes|y|1)$/i return false if self == false || self =~ /(false|f|no|n|0)$/i raise "invalid value for Boolean: '#{self}'" end |
#to_underscore ⇒ Object
37 38 39 40 41 42 43 44 45 46 47 |
# File 'lib/bcl/core_ext.rb', line 37 def to_underscore gsub('OpenStudio', 'Openstudio') .gsub('EnergyPlus', 'Energyplus') .gsub(/::/, '/') .gsub(/([A-Z]+)([A-Z][a-z])/, '\1_\2') .gsub(/([a-z\d])([A-Z])/, '\1_\2') .tr('-', '_') .tr(' ', '_') .squeeze('_') .downcase end |